@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2007 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,22 +14,22 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides debug utilities for Orca.  Debugging is managed by a debug
 level, which is held in the debugLevel field.  All other methods take
 a debug level, which is compared to the current debug level to
 determine if the content should be output."""
 
-__id__        = "$Id: debug.py 2535 2007-07-09 21:43:14Z wwalker $"
-__version__   = "$Revision: 2535 $"
-__date__      = "$Date: 2007-07-09 17:43:14 -0400 (Mon, 09 Jul 2007) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import sys
 import traceback
+import pyatspi
 
 # Used to turn off all debugging.
 #
@@ -144,17 +144,9 @@ def println(level, text = ""):
     - text: the text to print (default is a blank line)
     """
 
-    # Check to see if debugging is not enabled for the calling module.
-    #
-    frame = sys._getframe(1)
-    name = frame.f_globals["__name__"]
-    module = sys.modules[name]
-    if module.__dict__.has_key("_enableDebug") and not module._enableDebug:
-        return
-
     if level >= debugLevel:
         if debugFile:
-            debugFile.writelines([text,"\n"])
+            debugFile.writelines([text, "\n"])
         else:
             print text
 
@@ -175,12 +167,12 @@ def printObjectEvent(level, event, sourceInfo=None):
 
     level = max(level, eventDebugLevel)
 
-    text = "OBJECT EVENT: %-40s detail=(%d,%d)" \
-           % (event.type, event.detail1, event.detail2)
+    text = "OBJECT EVENT: %-40s detail=(%d,%d,%s)" \
+           % (event.type, event.detail1, event.detail2, event.any_data)
     println(level, text)
 
     if sourceInfo:
-        println(level, "             " + sourceInfo)
+        println(level, "             %s" % sourceInfo)
 
 def printInputEvent(level, string):
     """Prints out an input event.  The given level may be overridden
@@ -205,11 +197,60 @@ def printDetails(level, indent, accessible, includeApp=True):
     """
 
     if level >= debugLevel and accessible:
-        println(level, accessible.toString(indent, includeApp))
+        println(level, getAccessibleDetails(accessible, indent, includeApp))
+
+def getAccessibleDetails(acc, indent="", includeApp=True):
+    """Returns a string, suitable for printing, that describes the
+    given accessible.
+
+    Arguments:
+    - indent: A string to prefix the output with
+    - includeApp: If True, include information about the app
+                  for this accessible.
+    """
+
+    if includeApp:
+        app = acc.getApplication()
+        if app:
+            string = indent + "app.name='%s' " % app.name
+        else:
+            string = indent + "app=None "
+    else:
+        string = indent
+
+    # create the States string
+    stateSet = acc.getState()
+    states = stateSet.getStates()
+    state_strings = []
+    for state in states:
+        state_strings.append(pyatspi.stateToString(state))
+    state_string = ' '.join(state_strings)
+
+    # create the relations string
+    relations = acc.getRelationSet()
+    if relations:
+        relation_strings = []
+        for relation in relations:
+            relation_strings.append( \
+                          pyatspi.relationToString(relation.getRelationType()))
+        rel_string = ' '.join(relation_strings)
+    else:
+        rel_string = ''
+
+    try:
+        string += "name='%s' role='%s' state='%s' relations='%s'" \
+                  % (acc.name or 'None', acc.getRoleName(),
+                     state_string, rel_string)
+    except DataError:
+        string += "(exception fetching data)"
+
+    return string
+
 
 # The following code has been borrowed from the following URL:
 # 
-# http://www.dalkescientific.com/writings/diary/archive/2005/04/20/tracing_python_code.html
+# http://www.dalkescientific.com/writings/diary/archive/ \
+#                                     2005/04/20/tracing_python_code.html
 #
 import linecache
 
